@@ -59,6 +59,7 @@ enum {
   ARG_MIN_RESIDUAL_PARTITION_ORDER,
   ARG_MAX_RESIDUAL_PARTITION_ORDER,
   ARG_RICE_PARAMETER_SEARCH_DIST,
+  ARG_METADATA
 };
 
 static void		gst_flacenc_init		(FlacEnc *flacenc);
@@ -78,12 +79,13 @@ static GstElementStateReturn
 			gst_flacenc_change_state 	(GstElement *element);
 
 static FLAC__StreamEncoderWriteStatus 
-			gst_flacenc_write_callback 	(const FLAC__StreamEncoder *encoder, 
+			gst_flacenc_write_callback 	(const FLAC__SeekableStreamEncoder *encoder, 
 							 const FLAC__byte buffer[], unsigned bytes, 
 				    			 unsigned samples, unsigned current_frame, 
 							 void *client_data);
-static void 		gst_flacenc_metadata_callback 	(const FLAC__StreamEncoder *encoder, 
-							 const FLAC__StreamMetadata *metadata, 
+static FLAC__SeekableStreamEncoderSeekStatus
+			gst_flacenc_seek_callback       (const FLAC__SeekableStreamEncoder *encoder,
+			                                 FLAC__uint64 absolute_byte_offset,
 							 void *client_data);
 
 static GstElementClass *parent_class = NULL;
@@ -255,6 +257,9 @@ gst_flacenc_class_init (FlacEncClass *klass)
 		         "parameters, use best",
                        0, FLAC__MAX_RICE_PARTITION_ORDER, 
                        flacenc_params[DEFAULT_QUALITY].rice_parameter_search_dist, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_METADATA,
+    g_param_spec_boxed ("metadata", "Metadata", "Metadata to add to the stream,",
+                        GST_TYPE_CAPS, G_PARAM_READWRITE));		       
 
   gstelement_class->change_state = gst_flacenc_change_state;
 }
@@ -272,15 +277,28 @@ gst_flacenc_init (FlacEnc *flacenc)
 
   GST_FLAG_SET (flacenc, GST_ELEMENT_EVENT_AWARE);
 
-  flacenc->encoder = FLAC__stream_encoder_new();
+  flacenc->encoder = FLAC__seekable_stream_encoder_new();
 
-  FLAC__stream_encoder_set_write_callback (flacenc->encoder, 
+  FLAC__seekable_stream_encoder_set_write_callback (flacenc->encoder, 
 		                        gst_flacenc_write_callback);
-  FLAC__stream_encoder_set_metadata_callback (flacenc->encoder, 
-		                        gst_flacenc_metadata_callback);
-  FLAC__stream_encoder_set_client_data (flacenc->encoder, 
+  FLAC__seekable_stream_encoder_set_seek_callback (flacenc->encoder, 
+		                        gst_flacenc_seek_callback);
+  					
+  FLAC__seekable_stream_encoder_set_client_data (flacenc->encoder, 
 		                        flacenc);
 
+  flacenc->metadata = GST_CAPS_NEW (
+                  "flacenc_metadata",
+                  "application/x-gst-metadata",
+		    "comment",  GST_PROPS_STRING ("Track encoded with GStreamer"),
+                   "date",     GST_PROPS_STRING (""),
+		    "tracknum", GST_PROPS_STRING (""),
+		    "title",    GST_PROPS_STRING (""),
+		    "artist",   GST_PROPS_STRING (""),
+		    "album",    GST_PROPS_STRING (""),
+		    "genre",    GST_PROPS_STRING ("")
+                  );
+		  
   flacenc->negotiated = FALSE;
   flacenc->first = TRUE;
   flacenc->first_buf = NULL;
@@ -293,7 +311,7 @@ gst_flacenc_dispose (GObject *object)
 {
   FlacEnc *flacenc = GST_FLACENC (object);
 
-  FLAC__stream_encoder_delete (flacenc->encoder);
+  FLAC__seekable_stream_encoder_delete (flacenc->encoder);
   
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -311,12 +329,12 @@ gst_flacenc_sinkconnect (GstPad *pad, GstCaps *caps)
   gst_caps_get_int (caps, "channels", &flacenc->channels);
   gst_caps_get_int (caps, "depth", &flacenc->depth);
   gst_caps_get_int (caps, "rate", &flacenc->sample_rate);
-
-  FLAC__stream_encoder_set_bits_per_sample (flacenc->encoder, 
+  
+  FLAC__seekable_stream_encoder_set_bits_per_sample (flacenc->encoder, 
 		  			    flacenc->depth);
-  FLAC__stream_encoder_set_sample_rate (flacenc->encoder, 
+  FLAC__seekable_stream_encoder_set_sample_rate (flacenc->encoder, 
 		  			flacenc->sample_rate);
-  FLAC__stream_encoder_set_channels (flacenc->encoder, 
+  FLAC__seekable_stream_encoder_set_channels (flacenc->encoder, 
 		  		     flacenc->channels);
 
   flacenc->negotiated = TRUE;
@@ -331,9 +349,9 @@ gst_flacenc_update_quality (FlacEnc *flacenc, gint quality)
 
 #define DO_UPDATE(name, val, str) 	                    	\
   G_STMT_START {                                                \
-    if (FLAC__stream_encoder_get_##name (flacenc->encoder) != 	\
+    if (FLAC__seekable_stream_encoder_get_##name (flacenc->encoder) != 	\
         flacenc_params[quality].val) {        			\
-      FLAC__stream_encoder_set_##name (flacenc->encoder, 	\
+      FLAC__seekable_stream_encoder_set_##name (flacenc->encoder, 	\
           flacenc_params[quality].val);				\
       g_object_notify (G_OBJECT (flacenc), str);                \
     }                                                           \
@@ -360,15 +378,36 @@ gst_flacenc_update_quality (FlacEnc *flacenc, gint quality)
   return TRUE;
 }
 
+static FLAC__SeekableStreamEncoderSeekStatus
+gst_flacenc_seek_callback (const FLAC__SeekableStreamEncoder *encoder,
+			   FLAC__uint64 absolute_byte_offset,
+			   void *client_data)
+{
+FlacEnc *flacenc;
+GstEvent *event;
+
+  flacenc = GST_FLACENC (client_data);
+
+  if (flacenc->stopped) 
+    return FLAC__STREAM_ENCODER_OK;
+
+  event = gst_event_new_seek ((GstSeekType)(int)(GST_FORMAT_BYTES | GST_SEEK_METHOD_SET), absolute_byte_offset); 
+
+  if (event)
+    gst_pad_push (flacenc->srcpad, GST_BUFFER (event));
+ 
+  return  FLAC__STREAM_ENCODER_OK;
+}
+
 static FLAC__StreamEncoderWriteStatus 
-gst_flacenc_write_callback (const FLAC__StreamEncoder *encoder, 
+gst_flacenc_write_callback (const FLAC__SeekableStreamEncoder *encoder, 
 			    const FLAC__byte buffer[], unsigned bytes, 
 			    unsigned samples, unsigned current_frame, 
 			    void *client_data)
 {
   FlacEnc *flacenc;
   GstBuffer *outbuf;
-
+  
   flacenc = GST_FLACENC (client_data);
 
   if (flacenc->stopped) 
@@ -389,55 +428,51 @@ gst_flacenc_write_callback (const FLAC__StreamEncoder *encoder,
   return FLAC__STREAM_ENCODER_OK;
 }
 
-static void 
-gst_flacenc_metadata_callback (const FLAC__StreamEncoder *encoder, 
-		               const FLAC__StreamMetadata *metadata, 
-			       void *client_data)
+static void
+gst_flacenc_set_metadata (FlacEnc *flacenc, GstCaps *caps)
 {
-  GstEvent *event;
-  FlacEnc *flacenc;
+  GList *props;
+  GstPropsEntry *prop;
+  guint comments;
 
-  flacenc = GST_FLACENC (client_data);
+  g_return_if_fail (flacenc != NULL);
 
-  if (flacenc->stopped) 
-    return;
+  flacenc->meta = g_malloc (sizeof (FLAC__StreamMetadata **));
 
-  event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, 0, NULL);
-  gst_pad_push (flacenc->srcpad, GST_BUFFER (event));
+  flacenc->meta[0] = FLAC__metadata_object_new (FLAC__METADATA_TYPE_VORBIS_COMMENT);
 
-  if (flacenc->first_buf) {
-    const FLAC__uint64 samples = metadata->data.stream_info.total_samples;
-    const unsigned min_framesize = metadata->data.stream_info.min_framesize;
-    const unsigned max_framesize = metadata->data.stream_info.max_framesize;
+  comments = 0;
+  props = gst_caps_get_props (caps)->properties;
+  while (props) {
+    prop = (GstPropsEntry*)(props->data);
+    props = g_list_next(props);
 
-    guint8 *data = GST_BUFFER_DATA (flacenc->first_buf);
-    GstBuffer *outbuf = flacenc->first_buf;
+    if (gst_props_entry_get_type (prop) == GST_PROPS_STRING_TYPE) {
+      const gchar *name = gst_props_entry_get_name (prop);
+      const gchar *value;
+      gchar *data;
+      FLAC__StreamMetadata_VorbisComment_Entry commment_entry;
 
-    /* this looks evil but is actually how one is supposed to write
-     * the stream stats according to the FLAC examples */
+      gst_props_entry_get_string (prop, &value);
 
-    memcpy (&data[26], metadata->data.stream_info.md5sum, 16);
-    
-    data[21] = (data[21] & 0xf0) | 
-	       (FLAC__byte)((samples >> 32) & 0x0f);
-    data[22] = (FLAC__byte)((samples >> 24) & 0xff);
-    data[23] = (FLAC__byte)((samples >> 16) & 0xff);
-    data[24] = (FLAC__byte)((samples >> 8 ) & 0xff);
-    data[25] = (FLAC__byte)((samples      ) & 0xff);
+      if (!value || strlen (value) == 0)
+        continue;
 
-    data[12] = (FLAC__byte)((min_framesize >> 16) & 0xFF);
-    data[13] = (FLAC__byte)((min_framesize >> 8 ) & 0xFF);
-    data[14] = (FLAC__byte)((min_framesize      ) & 0xFF);
+      data = g_strdup_printf("%s=%s", name, value);
 
-    data[15] = (FLAC__byte)((max_framesize >> 16) & 0xFF);
-    data[16] = (FLAC__byte)((max_framesize >> 8 ) & 0xFF);
-    data[17] = (FLAC__byte)((max_framesize      ) & 0xFF);
+      commment_entry.length = GUINT32_TO_LE (strlen(data));
+      commment_entry.entry  = g_convert (data, commment_entry.length, "UTF8", "ISO-8859-1", NULL, NULL, NULL);
 
-    flacenc->first_buf = NULL;
-    gst_pad_push (flacenc->srcpad, outbuf);
+      FLAC__metadata_object_vorbiscomment_insert_comment (flacenc->meta[0], comments++, commment_entry, TRUE);
+      g_free (data);
+    }
   }
+
+  FLAC__seekable_stream_encoder_set_metadata(flacenc->encoder, flacenc->meta, 1);
 }
 
+
+
 static void
 gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
 {
@@ -457,7 +492,7 @@ gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
 
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_EOS:
-	FLAC__stream_encoder_finish(flacenc->encoder);
+	FLAC__seekable_stream_encoder_finish(flacenc->encoder);
       default:
 	gst_pad_event_default (pad, event);
         break;
@@ -476,12 +511,13 @@ gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
   insize = GST_BUFFER_SIZE (buf);
   samples = insize / ((depth+7)>>3);
 
-  if (FLAC__stream_encoder_get_state (flacenc->encoder) == 
-		  FLAC__STREAM_ENCODER_UNINITIALIZED) 
+  if (FLAC__seekable_stream_encoder_get_state (flacenc->encoder) == 
+		  FLAC__SEEKABLE_STREAM_ENCODER_UNINITIALIZED) 
   {
-    FLAC__StreamEncoderState state;
+    FLAC__SeekableStreamEncoderState state;
 
-    state = FLAC__stream_encoder_init (flacenc->encoder);
+    gst_flacenc_set_metadata (flacenc, flacenc->metadata);
+    state = FLAC__seekable_stream_encoder_init (flacenc->encoder);
     if (state != FLAC__STREAM_ENCODER_OK) {
       gst_element_error (GST_ELEMENT (flacenc),
 		         "could not initialize encoder (wrong parameters?)");
@@ -496,6 +532,7 @@ gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
   if (depth == 8) {
     gint8 *indata = (gint8 *) GST_BUFFER_DATA (buf);
     
+
     for (i=0; i<samples; i++) {
       data[i] = (FLAC__int32) *indata++;
     }
@@ -510,7 +547,7 @@ gst_flacenc_chain (GstPad *pad, GstBuffer *buf)
 
   gst_buffer_unref(buf);
 
-  res = FLAC__stream_encoder_process_interleaved (flacenc->encoder, 
+  res = FLAC__seekable_stream_encoder_process_interleaved (flacenc->encoder, 
 	                      (const FLAC__int32 *) data, samples / flacenc->channels);
 
   g_free (flacenc->data);
@@ -534,52 +571,55 @@ gst_flacenc_set_property (GObject *object, guint prop_id,
       gst_flacenc_update_quality (this, g_value_get_enum (value));
       break;
     case ARG_STREAMABLE_SUBSET:
-      FLAC__stream_encoder_set_streamable_subset (this->encoder, 
+      FLAC__seekable_stream_encoder_set_streamable_subset (this->encoder, 
 	                   g_value_get_boolean (value));
       break;
     case ARG_MID_SIDE_STEREO:
-      FLAC__stream_encoder_set_do_mid_side_stereo (this->encoder, 
+      FLAC__seekable_stream_encoder_set_do_mid_side_stereo (this->encoder, 
 	                   g_value_get_boolean (value));
       break;
     case ARG_LOOSE_MID_SIDE_STEREO:
-      FLAC__stream_encoder_set_loose_mid_side_stereo (this->encoder, 
+      FLAC__seekable_stream_encoder_set_loose_mid_side_stereo (this->encoder, 
 	                   g_value_get_boolean (value));
       break;
     case ARG_BLOCKSIZE:
-      FLAC__stream_encoder_set_blocksize (this->encoder, 
+      FLAC__seekable_stream_encoder_set_blocksize (this->encoder, 
 	                   g_value_get_uint (value));
       break;
     case ARG_MAX_LPC_ORDER:
-      FLAC__stream_encoder_set_max_lpc_order (this->encoder, 
+      FLAC__seekable_stream_encoder_set_max_lpc_order (this->encoder, 
 	                   g_value_get_uint (value));
       break;
     case ARG_QLP_COEFF_PRECISION:
-      FLAC__stream_encoder_set_qlp_coeff_precision (this->encoder, 
+      FLAC__seekable_stream_encoder_set_qlp_coeff_precision (this->encoder, 
 	                   g_value_get_uint (value));
       break;
     case ARG_QLP_COEFF_PREC_SEARCH:
-      FLAC__stream_encoder_set_do_qlp_coeff_prec_search (this->encoder, 
+      FLAC__seekable_stream_encoder_set_do_qlp_coeff_prec_search (this->encoder, 
 	                   g_value_get_boolean (value));
       break;
     case ARG_ESCAPE_CODING:
-      FLAC__stream_encoder_set_do_escape_coding (this->encoder, 
+      FLAC__seekable_stream_encoder_set_do_escape_coding (this->encoder, 
 	                   g_value_get_boolean (value));
       break;
     case ARG_EXHAUSTIVE_MODEL_SEARCH:
-      FLAC__stream_encoder_set_do_exhaustive_model_search (this->encoder, 
+      FLAC__seekable_stream_encoder_set_do_exhaustive_model_search (this->encoder, 
 	                   g_value_get_boolean (value));
       break;
     case ARG_MIN_RESIDUAL_PARTITION_ORDER:
-      FLAC__stream_encoder_set_min_residual_partition_order (this->encoder, 
+      FLAC__seekable_stream_encoder_set_min_residual_partition_order (this->encoder, 
 	                   g_value_get_uint (value));
       break;
     case ARG_MAX_RESIDUAL_PARTITION_ORDER:
-      FLAC__stream_encoder_set_max_residual_partition_order (this->encoder, 
+      FLAC__seekable_stream_encoder_set_max_residual_partition_order (this->encoder, 
 	                   g_value_get_uint (value));
       break;
     case ARG_RICE_PARAMETER_SEARCH_DIST:
-      FLAC__stream_encoder_set_rice_parameter_search_dist (this->encoder, 
+      FLAC__seekable_stream_encoder_set_rice_parameter_search_dist (this->encoder, 
 		                                 g_value_get_uint (value));
+      break; 
+    case ARG_METADATA:
+      this->metadata = g_value_get_boxed (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -601,51 +641,54 @@ gst_flacenc_get_property (GObject *object, guint prop_id,
       break;
     case ARG_STREAMABLE_SUBSET:
       g_value_set_boolean (value, 
-	      FLAC__stream_encoder_get_streamable_subset (this->encoder));
+	      FLAC__seekable_stream_encoder_get_streamable_subset (this->encoder));
       break;
     case ARG_MID_SIDE_STEREO:
       g_value_set_boolean (value, 
-	      FLAC__stream_encoder_get_do_mid_side_stereo (this->encoder));
+	      FLAC__seekable_stream_encoder_get_do_mid_side_stereo (this->encoder));
       break;
     case ARG_LOOSE_MID_SIDE_STEREO:
       g_value_set_boolean (value, 
-	      FLAC__stream_encoder_get_loose_mid_side_stereo (this->encoder));
+	      FLAC__seekable_stream_encoder_get_loose_mid_side_stereo (this->encoder));
       break;
     case ARG_BLOCKSIZE:
       g_value_set_uint (value, 
-	      FLAC__stream_encoder_get_blocksize (this->encoder));
+	      FLAC__seekable_stream_encoder_get_blocksize (this->encoder));
       break;
     case ARG_MAX_LPC_ORDER:
       g_value_set_uint (value, 
-	      FLAC__stream_encoder_get_max_lpc_order (this->encoder));
+	      FLAC__seekable_stream_encoder_get_max_lpc_order (this->encoder));
       break;
     case ARG_QLP_COEFF_PRECISION:
       g_value_set_uint (value, 
-	      FLAC__stream_encoder_get_qlp_coeff_precision (this->encoder));
+	      FLAC__seekable_stream_encoder_get_qlp_coeff_precision (this->encoder));
       break;
     case ARG_QLP_COEFF_PREC_SEARCH:
       g_value_set_boolean (value, 
-	      FLAC__stream_encoder_get_do_qlp_coeff_prec_search (this->encoder));
+	      FLAC__seekable_stream_encoder_get_do_qlp_coeff_prec_search (this->encoder));
       break;
     case ARG_ESCAPE_CODING:
       g_value_set_boolean (value, 
-	      FLAC__stream_encoder_get_do_escape_coding (this->encoder));
+	      FLAC__seekable_stream_encoder_get_do_escape_coding (this->encoder));
       break;
     case ARG_EXHAUSTIVE_MODEL_SEARCH:
       g_value_set_boolean (value, 
-	      FLAC__stream_encoder_get_do_exhaustive_model_search (this->encoder));
+	      FLAC__seekable_stream_encoder_get_do_exhaustive_model_search (this->encoder));
       break;
     case ARG_MIN_RESIDUAL_PARTITION_ORDER:
       g_value_set_uint (value, 
-	      FLAC__stream_encoder_get_min_residual_partition_order (this->encoder));
+	      FLAC__seekable_stream_encoder_get_min_residual_partition_order (this->encoder));
       break;
     case ARG_MAX_RESIDUAL_PARTITION_ORDER:
       g_value_set_uint (value, 
-	      FLAC__stream_encoder_get_max_residual_partition_order (this->encoder));
+	      FLAC__seekable_stream_encoder_get_max_residual_partition_order (this->encoder));
       break;
     case ARG_RICE_PARAMETER_SEARCH_DIST:
       g_value_set_uint (value, 
-	      FLAC__stream_encoder_get_rice_parameter_search_dist (this->encoder));
+	      FLAC__seekable_stream_encoder_get_rice_parameter_search_dist (this->encoder));
+      break;
+    case ARG_METADATA:
+      g_value_set_static_boxed (value, this->metadata);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -668,10 +711,10 @@ gst_flacenc_change_state (GstElement *element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
-      if (FLAC__stream_encoder_get_state (flacenc->encoder) != 
+      if (FLAC__seekable_stream_encoder_get_state (flacenc->encoder) != 
 		        FLAC__STREAM_ENCODER_UNINITIALIZED) {
         flacenc->stopped = TRUE;
-        FLAC__stream_encoder_finish (flacenc->encoder);
+        FLAC__seekable_stream_encoder_finish (flacenc->encoder);
       }
       flacenc->negotiated = FALSE;
       if (flacenc->first_buf)
@@ -679,6 +722,8 @@ gst_flacenc_change_state (GstElement *element)
       flacenc->first_buf = NULL;
       g_free (flacenc->data);
       flacenc->data = NULL;
+      FLAC__metadata_object_delete (flacenc->meta[0]);
+      g_free (flacenc->meta);
       break;
     case GST_STATE_READY_TO_NULL:
     default:
@@ -690,4 +735,3 @@ gst_flacenc_change_state (GstElement *element)
 
   return GST_STATE_SUCCESS;
 }
-
