@@ -48,6 +48,8 @@ struct _GstModPlug {
   GstElement element;
   GstPad *sinkpad, *srcpad;
   guint8 *buffer_in;
+  GstCaps *metadata;
+  GstCaps	*streaminfo;
   GstByteStream *bs;
 
   const gchar *songname;
@@ -68,9 +70,14 @@ struct _GstModPlug {
 
   guchar *audiobuffer;
   gint32 length;
-  gboolean restart;
+  guint state;
+  guint bitsPerSample;
+  gboolean need_discont;
+  gboolean eos;
   gint64 seek_at;
   guint64 total_samples;
+  guint64 song_size;
+  guint64 bytes_read;
 
   CSoundFile *mSoundFile;
 };
