@@ -168,7 +168,7 @@ gst_mpegaudio_class_init (GstMpegAudioClass *klass)
 
 }
 
-static GstPadConnectReturn
+static GstPadLinkReturn
 gst_mpegaudio_sinkconnect (GstPad *pad, GstCaps *caps)
 {
   GstMpegAudio *mpegaudio;
@@ -177,7 +177,7 @@ gst_mpegaudio_sinkconnect (GstPad *pad, GstCaps *caps)
   mpegaudio = GST_MPEGAUDIO (gst_pad_get_parent (pad));
 
   if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_CONNECT_DELAYED;
+    return GST_PAD_LINK_DELAYED;
 
   gst_caps_get_int (caps, "rate", &frequency);
   gst_caps_get_int (caps, "channels", &channels);
@@ -188,7 +188,7 @@ gst_mpegaudio_sinkconnect (GstPad *pad, GstCaps *caps)
 
   mpegaudio_sync_parms (mpegaudio->encoder);
 
-  return GST_PAD_CONNECT_OK;
+  return GST_PAD_LINK_OK;
 }
 
 static void
@@ -199,7 +199,7 @@ gst_mpegaudio_init (GstMpegAudio *mpegaudio)
 		  GST_PAD_TEMPLATE_GET (mpegaudio_sink_templ), "sink");
   gst_element_add_pad (GST_ELEMENT (mpegaudio), mpegaudio->sinkpad);
   gst_pad_set_chain_function (mpegaudio->sinkpad, gst_mpegaudio_chain);
-  gst_pad_set_connect_function (mpegaudio->sinkpad, gst_mpegaudio_sinkconnect);
+  gst_pad_set_link_function (mpegaudio->sinkpad, gst_mpegaudio_sinkconnect);
 
   mpegaudio->srcpad = gst_pad_new_from_template (
 		  GST_PAD_TEMPLATE_GET (mpegaudio_src_templ), "src");
